/* configureDialog.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "configureDialog.h"
#include "defaultForms.h"
#include "queryResult.h"
#include "database.h"
#include "string_utils.h"
#include "infoMessage.h"
#include "extract_words.h"
#include "format.h"
#include "sfClient.h"

#include <stdio.h>

ConfigureDialog::
ConfigureDialog(const Dataset &data, int form_id) :
  PalomaDialog(data), _form_id(form_id), 
  _menu("<configure>"),
  _cdrom_tab("CD-Rom"),
  _cd_cdrom_type_label("Soundfile type of CD-Rom device:"),
  _cd_cdrom_load_type_label("Load from CD-Rom as file type:"),
  _cd_cdrom_load_volume_label("Store loaded files in volume:"),
  _cd_cdrom_want_type_label("Encode from loaded type to file type:"),
  _cd_cdrom_want_volume_label("Store encoded files in volume:"),
  _cd_sfcomment_label("(See 'File types' tab for shell commands)"),
  _soundfile_tab("File types"),
  _soundfile_label("For soundfile type: "),
  _new_soundfile("Create type"),
  _delete_soundfile("Delete type"),
  _sf_play_command_label("Play command:"),
  _sf_play_response_label("Play response:"),
  _sf_verify_command_label("Verify command:"),
  _sf_verify_response_label("Verify response:"),
  _sf_convert_label("To convert to type: "),
  _sf_extensions_label("Filename extensions (space-delimited, include leading dot): "),
  _volume_tab("Volumes"),
  _volume_label("Volume: "),
  _new_volume("Create volume"),
  _delete_volume("Delete volume"),
  _vm_definition_label("Definition:"),
  _vm_bsize_label("Bucket size (MB):"),
  _vm_bcurrent_label("Current bucket number:"),
  _vm_bname_label("Bucket subdirectory name:"),
  _ok_button("OK"),
  _cancel_button("Cancel")  
{
  DefaultForms::check_default(_form_id, TI_configure);

  // Load up all of the soundfile definitions.
  QueryResult qr = music_db->sql_command
    ("SELECT soundfile_id FROM soundfile ORDER BY description");
  int num_rows = qr.get_num_rows();
  int r;
  for (r = 0; r < num_rows; r++) {
    int soundfile_id = qr.get_cell(r, 0).get_int();
    TableDataProxy sf(TI_soundfile, soundfile_id);
    string description = sf[SFFI_description].get_string();
    _sfdefs.insert(SFDefs::value_type(soundfile_id, sf));

    _soundfile_pick.add_choice(soundfile_id, description);
    _sf_convert_pick.add_choice(soundfile_id, description);
    _cd_cdrom_type_pick.add_choice(soundfile_id, description);
    _cd_cdrom_load_type_pick.add_choice(soundfile_id, description);
    _cd_cdrom_want_type_pick.add_choice(soundfile_id, description);

    // For each soundfile type, get all the extensions.
    QueryResult qr2 = music_db->sql_command
      ("SELECT extension FROM soundfile_exts"
       " WHERE soundfile_id=" + DataCell(soundfile_id));
    string extensions;
    int num_rows2 = qr2.get_num_rows();
    for (int r2 = 0; r2 < num_rows2; r2++) {
      extensions += " " + qr2.get_cell(r2, 0).get_string();
    }
    if (!extensions.empty()) {
      _sfextensions[soundfile_id].set_db_value(extensions.substr(1));
    } else {
      _sfextensions[soundfile_id].set_db_value(DataCell(""));
    }
  }
  _soundfile_pick.finish_choices();
  _sf_convert_pick.finish_choices();
  _cd_cdrom_type_pick.finish_choices();
  _cd_cdrom_load_type_pick.finish_choices();
  _cd_cdrom_want_type_pick.finish_choices();

  // Also fill up all the available conversion strings.
  qr = music_db->sql_command
    ("SELECT from_type, to_type, command FROM soundfile_convert");
  num_rows = qr.get_num_rows();
  for (r = 0; r < num_rows; r++) {
    int from_type = qr.get_cell(r, 0).get_int(-1);
    int to_type = qr.get_cell(r, 1).get_int(-1);

    if (from_type >= 0 && to_type >= 0) {
      // We convert any NULL values that might have come in from the
      // database to empty string.  This allows us to differentiate
      // later between commands that were not previously in the
      // database and those that were: if the db_value is NULL, it was
      // not in the database.

      // This distinction between NULL and empty string is meaningful
      // even if the particular SQL backend we are using does not make
      // this distinction, because it happens at a higher level than
      // the SQL backend.
      _sfconverts[from_type][to_type].set_db_value
	(DataCell(qr.get_cell(r, 2).get_string()));
    }
  }

  _current_sfdef = -1;
  _current_sfconvert = -1;
  new_sfdef();

  // Finally, get all the volume definitions.
  qr = music_db->sql_command
    ("SELECT volume_id FROM volumes ORDER BY description");
  num_rows = qr.get_num_rows();
  for (r = 0; r < num_rows; r++) {
    int volume_id = qr.get_cell(r, 0).get_int();
    TableDataProxy vm(TI_volumes, volume_id);
    string description = vm[VFI_description].get_string();
    _vmdefs.insert(VMDefs::value_type(volume_id, vm));

    _volume_pick.add_choice(volume_id, description);
    _cd_cdrom_load_volume_pick.add_choice(volume_id, description);
    _cd_cdrom_want_volume_pick.add_choice(volume_id, description);
  }
  _volume_pick.finish_choices();
  _cd_cdrom_load_volume_pick.finish_choices();
  _cd_cdrom_want_volume_pick.finish_choices();

  _current_vmdef = -1;
  new_vmdef();

  _cd_cdrom_type_pick.set_item(_data[TI_configure][CFI_cdrom_type].get_int());
  _cd_cdrom_load_type_pick.set_item(_data[TI_configure][CFI_cdrom_load_type].get_int());
  _cd_cdrom_load_volume_pick.set_item(_data[TI_configure][CFI_cdrom_load_volume].get_int());
  _cd_cdrom_want_type_pick.set_item(_data[TI_configure][CFI_cdrom_want_type].get_int());
  _cd_cdrom_want_volume_pick.set_item(_data[TI_configure][CFI_cdrom_want_volume].get_int());

  /*
  if (_form_id >= 0) {
    _form.layout_form(_form_id, _data);
    _form.show();
  }
  */

  layout_window();
  setup();
}

ConfigureDialog::
~ConfigureDialog() {
}

void ConfigureDialog::
record_db() {
  music_db->begin();

  _data[TI_configure][CFI_cdrom_type] = DataCell(_cd_cdrom_type_pick.get_item());
  _data[TI_configure][CFI_cdrom_load_type] = DataCell(_cd_cdrom_load_type_pick.get_item());
  _data[TI_configure][CFI_cdrom_load_volume] = DataCell(_cd_cdrom_load_volume_pick.get_item());
  _data[TI_configure][CFI_cdrom_want_type] = DataCell(_cd_cdrom_want_type_pick.get_item());
  _data[TI_configure][CFI_cdrom_want_volume] = DataCell(_cd_cdrom_want_volume_pick.get_item());
  //  _form.update();

  // Record the configure record itself.
  _data[TI_configure].record_db();

  record_sfdef();
  record_vmdef();

  music_db->commit();

  // Now tell the sfserver to re-read, just in case we changed
  // something.
  SFClient sf;
  sf.reread_config();
}
 
void ConfigureDialog::
record_sfdef() {
  save_sfdef();

  SFDefs::iterator sfi;
  for (sfi = _sfdefs.begin(); sfi != _sfdefs.end(); ++sfi) {
    if ((*sfi).second.has_table()) {
      (*sfi).second.record_db();
    } else {
      // If the TableDataProxy associated with the sfdef has no table,
      // it's a cue that we've deleted the sfdef.  So now remove it
      // from the database.
      int soundfile_id = (*sfi).first;

      // Make sure we haven't already removed it before.
      if (_removed_sfdefs.count(soundfile_id) == 0) {
	_removed_sfdefs.insert(soundfile_id);

	// Remove any references from the configure record.
	if (_data[TI_configure][CFI_cdrom_type].get_int(-1) == soundfile_id) {
	  _data[TI_configure][CFI_cdrom_type] = DataCell();
	}
	if (_data[TI_configure][CFI_cdrom_load_type].get_int(-1) == soundfile_id) {
	  _data[TI_configure][CFI_cdrom_load_type] = DataCell();
	}
	if (_data[TI_configure][CFI_cdrom_want_type].get_int(-1) == soundfile_id) {
	  _data[TI_configure][CFI_cdrom_want_type] = DataCell();
	}
	_data[TI_configure].record_db();
	
	// And from the convert and file extensions records.
	music_db->sql_command
	  ("DELETE FROM soundfile_convert"
	   " WHERE from_type=" + DataCell(soundfile_id) +
	   " OR to_type=" + DataCell(soundfile_id));
	music_db->sql_command
	  ("DELETE FROM soundfile_exts"
	   " WHERE soundfile_id=" + DataCell(soundfile_id));
	
	// And remove the soundfile definition itself.
	music_db->sql_command
	  ("DELETE FROM soundfile"
	   " WHERE soundfile_id=" + DataCell(soundfile_id));
      }
    }
  }

  record_convert();
  record_extensions();
}

void ConfigureDialog::
record_convert() {
  // Record to the database all of the convert strings associated with
  // all of the soundfile types.
  SFConverts::iterator sci;
  for (sci = _sfconverts.begin(); sci != _sfconverts.end(); ++sci) {
    int from_type = (*sci).first;
    SFConvertsInternal &in = (*sci).second;

    // Make sure we haven't just removed this type.
    if (_removed_sfdefs.count(from_type) == 0) {
      SFConvertsInternal::iterator scii;
      for (scii = in.begin(); scii != in.end(); ++scii) {
	int to_type = (*scii).first;
	TableData::Value &val = (*scii).second;

	// And make sure we haven't just removed this type, either.
	if (_removed_sfdefs.count(to_type) == 0) {
	  if (val.has_changed()) {
	    string command = trim_blanks(val._current.get_string());
	    
	    if (val._db.is_null()) {
	      if (!command.empty()) {
		// The value was NULL before, but it is non-NULL now.
		// That means it wasn't previously in the database, but
		// now it is.
		music_db->sql_command
		  ("INSERT INTO soundfile_convert (from_type, to_type, command)"
		   " VALUES (" +
		   DataCell(from_type) + ", " +
		   DataCell(to_type) + ", " +
		   DataCell(command) + ")");
	      }
	      
	    } else if (command.empty()) {
	      // The value was non-NULL before, but it is empty now.  That
	      // means it was previously in the database, but the user
	      // wiped it out.
	      music_db->sql_command
		("DELETE FROM soundfile_convert"
		 " WHERE from_type=" + DataCell(from_type) +
		 " AND to_type=" + DataCell(to_type));
	      
	    } else {
	      // The value was non-NULL before, and it is still non-NULL.
	      // The user just changed the value from one string to
	      // another.
	      music_db->sql_command
		("UPDATE soundfile_convert"
		 " SET command=" + DataCell(command) +
		 " WHERE from_type=" + DataCell(from_type) +
		 " AND to_type=" + DataCell(to_type));
	    }
	    
	    val.set_db_value(DataCell(command, !command.empty()));
	  }
	}
      }
    }
  }
}

void ConfigureDialog::
record_extensions() {
  // Record to the database all of the filename extensions associated
  // with each of the soundfile types.
  SFExtensions::iterator sei;
  for (sei = _sfextensions.begin(); sei != _sfextensions.end(); ++sei) {
    int soundfile_id = (*sei).first;
    TableData::Value &val = (*sei).second;

    // Make sure we haven't just removed this type.
    if (_removed_sfdefs.count(soundfile_id) == 0) {
      if (val.has_changed()) {
	vector<string> extensions = 
	  extract_words(val._current.get_string());

	// First, delete whatever was there before.  This is a minor
	// race condition with other processes.  Maybe we should lock
	// the table here?
	music_db->sql_command
	  ("DELETE FROM soundfile_exts"
	   " WHERE soundfile_id=" + DataCell(soundfile_id));

	// Now, insert them all back again.
	for (int i = 0; i < extensions.size(); i++) {
	  music_db->sql_command
	    ("INSERT INTO soundfile_exts (soundfile_id, extension) VALUES (" +
	     DataCell(soundfile_id) + ", " + 
	     DataCell(extensions[i]) + ")");
	}

	val._db = val._current;
      }
    }
  }
}

void ConfigureDialog::
new_soundfile_clicked() {
  string description = _soundfile_pick.get_entry()->get_text();
  int soundfile_id = music_db->get_next_sequence("soundfile_soundfile_id_seq");

  TableDataProxy sf(TI_soundfile);
  sf.set_primary_key(soundfile_id, true);
  sf[SFFI_description] = DataCell(description);
  _sfdefs.insert(SFDefs::value_type(soundfile_id, sf));

  _soundfile_pick.add_choice(soundfile_id, description);
  _sf_convert_pick.add_choice(soundfile_id, description);
  _cd_cdrom_type_pick.add_choice(soundfile_id, description);
  _cd_cdrom_load_type_pick.add_choice(soundfile_id, description);
  _cd_cdrom_want_type_pick.add_choice(soundfile_id, description);
  _soundfile_pick.finish_choices();
  _sf_convert_pick.finish_choices();
  _cd_cdrom_type_pick.finish_choices();
  _cd_cdrom_load_type_pick.finish_choices();
  _cd_cdrom_want_type_pick.finish_choices();

  _soundfile_pick.set_item(soundfile_id);
  new_sfdef();
}
 
void ConfigureDialog::
record_vmdef() {
  save_vmdef();

  VMDefs::iterator vmi;
  for (vmi = _vmdefs.begin(); vmi != _vmdefs.end(); ++vmi) {
    if ((*vmi).second.has_table()) {
      (*vmi).second.record_db();
    } else {
      // If the TableDataProxy associated with the vmdef has no table,
      // it's a cue that we've deleted the vmdef.  So now remove it
      // from the database.
      int volume_id = (*vmi).first;

      // Make sure we haven't already removed it before.
      if (_removed_vmdefs.count(volume_id) == 0) {
	_removed_vmdefs.insert(volume_id);

	// Remove any references from the configure record.
	if (_data[TI_configure][CFI_cdrom_load_volume].get_int(-1) == volume_id) {
	  _data[TI_configure][CFI_cdrom_load_volume] = DataCell();
	}
	if (_data[TI_configure][CFI_cdrom_want_volume].get_int(-1) == volume_id) {
	  _data[TI_configure][CFI_cdrom_want_volume] = DataCell();
	}
	_data[TI_configure].record_db();

	bool delete_songs = _vmdelete_songs[volume_id];
	if (delete_songs) {

	  // We should also delete the CD records here.

	  music_db->sql_command
	    ("DELETE FROM cd_tracks"
	     " WHERE cd_tracks.song_id=songs.song_id"
	     " AND songs.volume_id=" + DataCell(volume_id));
	  music_db->sql_command
	    ("DELETE FROM songs"
	     " WHERE volume_id=" + DataCell(volume_id));

	} else {
	  music_db->sql_command
	    ("UPDATE songs SET volume_id=NULL, loaded_time=NULL"
	     " WHERE volume_id=" + DataCell(volume_id));
	}
	
	music_db->sql_command
	  ("DELETE FROM volumes"
	   " WHERE volume_id=" + DataCell(volume_id));
      }
    }
  }
}


void ConfigureDialog::
delete_soundfile_clicked() {
  int soundfile_id = _soundfile_pick.get_item(-1);
  if (soundfile_id >= 0) {
    string description = _soundfile_pick.get_entry()->get_text();
    InfoMessage *im =
      new InfoMessage("Permamently remove definitions for the\n"
		      "soundfile type " + description + "?",
		      InfoMessage::IM_ok | InfoMessage::IM_cancel);
    add_child_dialog(im);
    
    connect_to_method(im->ok_clicked, this, &delete_soundfile_okayed,
		      soundfile_id);
  }
}

void ConfigureDialog::
delete_soundfile_okayed(InfoMessage *, int soundfile_id) {
  _soundfile_pick.remove_item(soundfile_id);
  _sf_convert_pick.remove_item(soundfile_id);
  _cd_cdrom_type_pick.remove_item(soundfile_id);
  _cd_cdrom_load_type_pick.remove_item(soundfile_id);
  _cd_cdrom_want_type_pick.remove_item(soundfile_id);

  // We mark that the soundfile def is to be deleted by flagging the
  // sfdata with a TableDataProxy that has no table.  The actual
  // deletion will occur when the user saves the dialog in
  // record_db().
  _sfdefs[soundfile_id] = TableDataProxy();
}

void ConfigureDialog::
new_volume_clicked() {
  string description = _volume_pick.get_entry()->get_text();
  int volume_id = music_db->get_next_sequence("volumes_volume_id_seq");

  TableDataProxy vm(TI_volumes);
  vm.set_primary_key(volume_id, true);
  vm[VFI_description] = DataCell(description);
  _vmdefs.insert(VMDefs::value_type(volume_id, vm));

  _volume_pick.add_choice(volume_id, description);
  _cd_cdrom_load_volume_pick.add_choice(volume_id, description);
  _cd_cdrom_want_volume_pick.add_choice(volume_id, description);
  _volume_pick.finish_choices();
  _cd_cdrom_load_volume_pick.finish_choices();
  _cd_cdrom_want_volume_pick.finish_choices();

  _volume_pick.set_item(volume_id);
  new_vmdef();
}


void ConfigureDialog::
delete_volume_clicked() {
  int volume_id = _volume_pick.get_item(-1);
  if (volume_id >= 0) {
    string description = _volume_pick.get_entry()->get_text();
    
    QueryResult qr = music_db->sql_command
      ("SELECT COUNT(*) FROM songs"
       " WHERE volume_id=" + DataCell(volume_id));
    assert(qr.get_num_rows() > 0);
    int num_songs = qr.get_cell(0, 0).get_int();

    string warning;
    int buttons = InfoMessage::IM_ok | InfoMessage::IM_cancel;

    if (num_songs > 0) {
      warning = "There are " + format_int(num_songs) + 
	" songs that reference this volume.\n"
	"If you remove the volume, these songs will no longer\n"
	"be playable, although they will remain in the database\n"
	"unless you check the button below.\n\n"
	"In either case, removing the volume will not attempt\n"
	"to delete the actual files stored on the disk.  Only\n"
	"the contents of the SQL database will be affected.\n\n";
      buttons |= InfoMessage::IM_check;
    }

    warning += "Are you sure you want to permanently remove\n" + description 
      + "?";

    InfoMessage *im =
      new InfoMessage(warning, buttons,
		      "Also remove the songs from the database");
    add_child_dialog(im);
    
    connect_to_method(im->ok_clicked, this, &delete_volume_okayed,
		      volume_id);
  }
}

void ConfigureDialog::
delete_volume_okayed(InfoMessage *dialog, int volume_id) {
  bool delete_songs = dialog->_check.get_active();

  _volume_pick.remove_item(volume_id);
  _cd_cdrom_load_volume_pick.remove_item(volume_id);
  _cd_cdrom_want_volume_pick.remove_item(volume_id);

  // We mark that the volume def is to be deleted by flagging the
  // vmdata with a TableDataProxy that has no table.  The actual
  // deletion will occur when the user saves the dialog in
  // record_db().
  _vmdefs[volume_id] = TableDataProxy();
  _vmdelete_songs[volume_id] = delete_songs;
}

void ConfigureDialog::
ok_button_clicked() {
  record_db();

  accepted();
  delete_self();
}

void ConfigureDialog::
cancel_button_clicked() {
  delete_self();
}

void ConfigureDialog::
new_sfdef() {
  // This is called to load the values for a new soundfile definition
  // into the fields on the window.  It's called whenever the user
  // picks a new soundfile definition from the combo box.

  save_sfdef();

  _current_sfdef = _soundfile_pick.get_item(-1);

  if (_current_sfdef >= 0) {
    TableDataProxy sf = _sfdefs[_current_sfdef];
    _sf_play_command_entry.set_text(sf[SFFI_play_command].get_string());
    _sf_play_response_entry.set_text(sf[SFFI_play_response].get_string());
    _sf_verify_command_entry.set_text(sf[SFFI_verify_command].get_string());
    _sf_verify_response_entry.set_text(sf[SFFI_verify_response].get_string());
    _sf_extensions_entry.set_text(_sfextensions[_current_sfdef]._current.get_string());
    _sf_play_command_entry.set_sensitive(true);
    _sf_play_response_entry.set_sensitive(true);
    _sf_verify_command_entry.set_sensitive(true);
    _sf_verify_response_entry.set_sensitive(true);
    _sf_extensions_entry.set_sensitive(true);
    _sf_convert_pick.get_entry()->set_sensitive(true);
    _new_soundfile.set_sensitive(false);
    _delete_soundfile.set_sensitive(true);
  } else {
    _sf_play_command_entry.set_text("");
    _sf_play_response_entry.set_text("");
    _sf_verify_command_entry.set_text("");
    _sf_verify_response_entry.set_text("");
    _sf_extensions_entry.set_text("");
    _sf_convert_pick.get_entry()->set_text("");
    _sf_play_command_entry.set_sensitive(false);
    _sf_play_response_entry.set_sensitive(false);
    _sf_verify_command_entry.set_sensitive(false);
    _sf_verify_response_entry.set_sensitive(false);
    _sf_extensions_entry.set_sensitive(false);
    _sf_convert_pick.get_entry()->set_sensitive(false);
    _new_soundfile.set_sensitive(true);
    _delete_soundfile.set_sensitive(false);
  }

  _current_sfconvert = -1;
  new_sfconvert();
}

void ConfigureDialog::
save_sfdef() {
  // This just gets the values off of the fields and stores them in
  // our internal structures, in preparation for either writing to the
  // database or refilling the fields with the values for a different
  // soundfile type.

  save_sfconvert();
  if (_current_sfdef >= 0) {
    TableDataProxy sf = _sfdefs[_current_sfdef];
    sf[SFFI_play_command] = DataCell(_sf_play_command_entry.get_text());
    sf[SFFI_play_response] = DataCell(_sf_play_response_entry.get_text());
    sf[SFFI_verify_command] = DataCell(_sf_verify_command_entry.get_text());
    sf[SFFI_verify_response] = DataCell(_sf_verify_response_entry.get_text());
    _sfextensions[_current_sfdef]._current = DataCell(_sf_extensions_entry.get_text());
  }
}

void ConfigureDialog::
new_sfconvert() {
  save_sfconvert();
  _current_sfconvert = _sf_convert_pick.get_item(-1);

  if (_current_sfconvert >= 0) {
    DataCell value = _sfconverts[_current_sfdef][_current_sfconvert]._current;
    _sf_convert_entry.set_text(value.get_string());
    _sf_convert_entry.set_sensitive(true);
  } else {
    _sf_convert_entry.set_text("");
    _sf_convert_entry.set_sensitive(false);
  }
}

void ConfigureDialog::
save_sfconvert() {
  if (_current_sfdef >= 0 && _current_sfconvert >= 0) {
    _sfconverts[_current_sfdef][_current_sfconvert]._current =
      DataCell(_sf_convert_entry.get_text());
  }
}

void ConfigureDialog::
new_vmdef() {
  // This is called to load the values for a new volume definition
  // into the fields on the window.  It's called whenever the user
  // picks a new volume definition from the combo box.

  save_vmdef();

  _current_vmdef = _volume_pick.get_item(-1);

  if (_current_vmdef >= 0) {
    TableDataProxy vm = _vmdefs[_current_vmdef];
    _vm_definition_entry.set_text(vm[VFI_definition].get_string());
    _vm_bsize_entry.set_text(vm[VFI_bucket_size].get_text());
    _vm_bcurrent_entry.set_text(vm[VFI_current_bucket].get_text());
    _vm_bname_entry.set_text(vm[VFI_bucket_name].get_string());
    _vm_definition_entry.set_sensitive(true);
    _vm_bsize_entry.set_sensitive(true);
    _vm_bcurrent_entry.set_sensitive(true);
    _vm_bname_entry.set_sensitive(true);
    _new_volume.set_sensitive(false);
    _delete_volume.set_sensitive(true);
  } else {
    _vm_definition_entry.set_text("");
    _vm_bsize_entry.set_text("");
    _vm_bcurrent_entry.set_text("");
    _vm_bname_entry.set_text("");
    _vm_definition_entry.set_sensitive(false);
    _vm_bsize_entry.set_sensitive(false);
    _vm_bcurrent_entry.set_sensitive(false);
    _vm_bname_entry.set_sensitive(false);
    _new_volume.set_sensitive(true);
    _delete_volume.set_sensitive(false);
  }
}

void ConfigureDialog::
save_vmdef() {
  // This just gets the values off of the fields and stores them in
  // our internal structures, in preparation for either writing to the
  // database or refilling the fields with the values for a different
  // volume type.
  if (_current_vmdef >= 0) {
    TableDataProxy vm = _vmdefs[_current_vmdef];
    vm[VFI_definition] = DataCell(_vm_definition_entry.get_text());
    vm[VFI_bucket_size] = DataCell(atoi(_vm_bsize_entry.get_text().c_str()));
    vm[VFI_current_bucket] = DataCell(atoi(_vm_bcurrent_entry.get_text().c_str()));
    vm[VFI_bucket_name] = DataCell(_vm_bname_entry.get_text());
  }
}


void ConfigureDialog::
layout_window() {
  set_title("Configure Paloma");
  set_default_size(500, 0);

  // The menu bar.
  _menubar = _menu.get_menubar_widget("");
  assert(_menubar.get_object() != NULL);

  _menu.create_item("/File/Save", "", "",
		    ItemFactoryConnector<ConfigureDialog, int>(this, &record_db));
  _menu.create_item("/File/Close", "", "",
		    ItemFactoryConnector<ConfigureDialog, int>(this, &ok_button_clicked));


  vbox_.pack_start(*_menubar.get_object(), false, false);
  _menubar.get_object()->show();

  // The box for the notebook.
  vbox_.pack_start(_box1);
  _box1.show();
  _box1.set_border_width(8);

  _box1.pack_start(_notebook);
  _notebook.show();

  layout_soundfile_page();
  layout_volume_page();
  layout_cdrom_page();

  // The OK button.
  _ok_button.set_usize(80, 36);
  action_area_.pack_start(_ok_button, true, false, 0);
  _ok_button.set_flags(GTK_CAN_DEFAULT);
  _ok_button.grab_default();
  _ok_button.show();
  connect_to_method(_ok_button.clicked, this, &ok_button_clicked);
  
  // The Cancel button.
  _cancel_button.set_usize(80, 36);
  action_area_.pack_start(_cancel_button, true, false, 0);
  _cancel_button.set_flags(GTK_CAN_DEFAULT);
  _cancel_button.show();
  connect_to_method(_cancel_button.clicked, this, &cancel_button_clicked);
}

void ConfigureDialog::
layout_cdrom_page() {
  _notebook.append_page(_cdrom_box, _cdrom_tab);
  _cdrom_box.show();
  _cdrom_box.set_border_width(8);

  // CD-Rom soundfile type.
  _cd_cdrom_type_label.set_alignment(0, 0);
  _cdrom_box.pack_start(_cd_cdrom_type_label, false, false, 0);
  _cdrom_box.pack_start(_cd_cdrom_type_pick, false, false, 0);
  _cd_cdrom_type_label.show();
  _cd_cdrom_type_pick.show();

  // CD-Rom load soundfile type.
  _cd_cdrom_load_type_label.set_alignment(0, 0);
  _cdrom_box.pack_start(_cd_cdrom_load_type_label, false, false, 0);
  _cdrom_box.pack_start(_cd_cdrom_load_type_pick, false, false, 0);
  _cd_cdrom_load_type_label.show();
  _cd_cdrom_load_type_pick.show();

  // CD-Rom load volume.
  _cd_cdrom_load_volume_label.set_alignment(0, 0);
  _cdrom_box.pack_start(_cd_cdrom_load_volume_label, false, false, 0);
  _cdrom_box.pack_start(_cd_cdrom_load_volume_pick, false, false, 0);
  _cd_cdrom_load_volume_label.show();
  _cd_cdrom_load_volume_pick.show();

  // CD-Rom want soundfile type.
  _cd_cdrom_want_type_label.set_alignment(0, 0);
  _cdrom_box.pack_start(_cd_cdrom_want_type_label, false, false, 0);
  _cdrom_box.pack_start(_cd_cdrom_want_type_pick, false, false, 0);
  _cd_cdrom_want_type_label.show();
  _cd_cdrom_want_type_pick.show();

  // CD-Rom want volume.
  _cd_cdrom_want_volume_label.set_alignment(0, 0);
  _cdrom_box.pack_start(_cd_cdrom_want_volume_label, false, false, 0);
  _cdrom_box.pack_start(_cd_cdrom_want_volume_pick, false, false, 0);
  _cd_cdrom_want_volume_label.show();
  _cd_cdrom_want_volume_pick.show();

  // A helpful tip for the user.
  _cdrom_box.pack_start(_cd_sfcomment_label, true, true, 16);
  _cd_sfcomment_label.show();
}

void ConfigureDialog::
layout_soundfile_page() {
  _notebook.append_page(_soundfile_box, _soundfile_tab);
  _soundfile_box.show();
  _soundfile_box.set_border_width(8);

  _soundfile_box.pack_start(_sf_cbox, false, false);
  _sf_cbox.show();

  _sf_cbox.pack_start(_soundfile_label, false, false);
  _soundfile_label.show();
  _sf_cbox.pack_start(_soundfile_pick, true, true);
  _soundfile_pick.show();
  _soundfile_pick.set_value_in_list(false, false);
  connect_to_method(_soundfile_pick.changed_item, this, &new_sfdef);
  _sf_cbox.pack_start(_new_soundfile, true, false, 4);
  _new_soundfile.show();
  connect_to_method(_new_soundfile.clicked, this, &new_soundfile_clicked);
  _sf_cbox.pack_start(_delete_soundfile, true, false);
  _delete_soundfile.show();
  connect_to_method(_delete_soundfile.clicked, this, &delete_soundfile_clicked);

  // Play command, label and entry.
  _sf_play_command_label.set_alignment(0, 0);
  _soundfile_box.pack_start(_sf_play_command_label, false, false, 0);
  _soundfile_box.pack_start(_sf_play_command_entry, false, false, 0);
  _sf_play_command_label.show();
  _sf_play_command_entry.show();

  // Play response, label and entry.
  _sf_play_response_label.set_alignment(0, 0);
  _soundfile_box.pack_start(_sf_play_response_label, false, false, 0);
  _soundfile_box.pack_start(_sf_play_response_entry, false, false, 0);
  _sf_play_response_label.show();
  _sf_play_response_entry.show();

  // Verify command, label and entry.
  _sf_verify_command_label.set_alignment(0, 0);
  _soundfile_box.pack_start(_sf_verify_command_label, false, false, 0);
  _soundfile_box.pack_start(_sf_verify_command_entry, false, false, 0);
  _sf_verify_command_label.show();
  _sf_verify_command_entry.show();

  // Verify response, label and entry.
  _sf_verify_response_label.set_alignment(0, 0);
  _soundfile_box.pack_start(_sf_verify_response_label, false, false, 0);
  _soundfile_box.pack_start(_sf_verify_response_entry, false, false, 0);
  _sf_verify_response_label.show();
  _sf_verify_response_entry.show();

  // And the list of extensions.  This was loaded from the database
  // and artificially concatenated into a single string, for the
  // user's editing convenience.
  _sf_extensions_label.set_alignment(0, 0);
  _soundfile_box.pack_start(_sf_extensions_label, false, false, 0);
  _soundfile_box.pack_start(_sf_extensions_entry, false, false, 0);
  _sf_extensions_label.show();
  _sf_extensions_entry.show();

  // Now the "convert-to-this-type" entry.  This comes along with a
  // pick box to pick the type to convert to, as well.
  _soundfile_box.pack_start(_sf_convert_box, true, true, 2);
  _sf_convert_box.show();
  _sf_convert_box.pack_start(_sf_convert_label, false, false);
  _sf_convert_label.show();
  _sf_convert_box.pack_start(_sf_convert_pick, false, false);
  _sf_convert_pick.show();
  connect_to_method(_sf_convert_pick.changed_item, this, &new_sfconvert);
  _soundfile_box.pack_start(_sf_convert_entry);
  _sf_convert_entry.show();
}

void ConfigureDialog::
layout_volume_page() {
  _notebook.append_page(_volume_box, _volume_tab);
  _volume_box.show();
  _volume_box.set_border_width(8);

  _volume_box.pack_start(_vm_cbox, false, false);
  _vm_cbox.show();

  _vm_cbox.pack_start(_volume_label, false, false);
  _volume_label.show();
  _vm_cbox.pack_start(_volume_pick, true, true);
  _volume_pick.show();
  _volume_pick.set_value_in_list(false, false);
  connect_to_method(_volume_pick.changed_item, this, &new_vmdef);
  _vm_cbox.pack_start(_new_volume, true, false, 4);
  _new_volume.show();
  connect_to_method(_new_volume.clicked, this, &new_volume_clicked);
  _vm_cbox.pack_start(_delete_volume, true, false);
  _delete_volume.show();
  connect_to_method(_delete_volume.clicked, this, &delete_volume_clicked);

  // Volume definition, label and entry.
  _vm_definition_label.set_alignment(0, 0);
  _volume_box.pack_start(_vm_definition_label, false, false, 0);
  _volume_box.pack_start(_vm_definition_entry, false, false, 0);
  _vm_definition_label.show();
  _vm_definition_entry.show();

  // Bucket size, label and entry.
  _vm_bsize_label.set_alignment(0, 0);
  _volume_box.pack_start(_vm_bsize_label, false, false, 0);
  _volume_box.pack_start(_vm_bsize_entry, false, false, 0);
  _vm_bsize_label.show();
  _vm_bsize_entry.show();

  // Current bucket number, label and entry.
  _vm_bcurrent_label.set_alignment(0, 0);
  _volume_box.pack_start(_vm_bcurrent_label, false, false, 0);
  _volume_box.pack_start(_vm_bcurrent_entry, false, false, 0);
  _vm_bcurrent_label.show();
  _vm_bcurrent_entry.show();

  // Bucket subdirectory name, label and entry.
  _vm_bname_label.set_alignment(0, 0);
  _volume_box.pack_start(_vm_bname_label, false, false, 0);
  _volume_box.pack_start(_vm_bname_entry, false, false, 0);
  _vm_bname_label.show();
  _vm_bname_entry.show();
}
