/* paloma.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "paloma.h"
#include "cdReader.h"
#include "cdDialog.h"
#include "artistLookup.h"
#include "cdLookup.h"
#include "playlist.h"
#include "playlistDialog.h"
#include "queryForm.h"
#include "dataCell.h"
#include "database.h"
#include "get_basename.h"
#include "configureDialog.h"
#include "importDialog.h"
#include "exportDialog.h"

#include "dove.xpm"

Paloma *paloma = (Paloma *)NULL;

Paloma::
Paloma(const Dataset &data, const string &title,
       const vector<string> &cds, bool show_dove) : 
  _data(data),
  _title(title),
  _show_dove(show_dove),
  _menu("<main>")
{
  layout_window(cds);
  setup();

  // We have to wait until after we've shown the window to make the
  // pixmap(s), because we need the Gtk_Window object to figure out
  // the bit depth, etc.
  make_pixmaps();
}

Paloma::
~Paloma() {
  gtk_exit(0);
}

void Paloma::
menu_scan_cd(string device_name) {
#ifdef HAVE_CDAUDIO
  CDReader *cdr = new CDReader(_data, device_name);
  if (cdr->has_disc()) {
    CDDialog::make_dialog(_data, cdr);
  } else {
    delete cdr;
  }
#endif
}

void Paloma::
menu_import() {
  new ImportDialog(_data);
}

void Paloma::
menu_export_cds() {
  Playlist *playlist = new Playlist;
  playlist->append_data(music_db->sql_command("SELECT cd_id FROM cds"));
  new ExportDialog(_data, playlist, 0, true);
}

void Paloma::
menu_export_songs() {
  Playlist *playlist = new Playlist;
  playlist->append_data(music_db->sql_command("SELECT song_id FROM songs"));
  new ExportDialog(_data, playlist, 0, true);
}

void Paloma::
menu_configure() {
  new ConfigureDialog(_data, -1);
}

void Paloma::
menu_quit() {
  delete_self();
}

void Paloma::
menu_lookup_artists() {
  new ArtistLookup(_data, true);
}

void Paloma::
menu_lookup_cds() {
  new CDLookup(_data, true);
}

void Paloma::
menu_lookup_songs() {
  PlaylistDialog::make_dialog
    (_data, -1, 
     "SELECT song_id, performer, name"
     " FROM songs ORDER BY name");
}

void Paloma::
menu_begin_query(int form_id) {
  new QueryForm(_data, form_id);
}

void Paloma::
make_pixmaps() {
  if (_show_dove) {
    Gdk_Window window = get_window();

    _dove_gdk_pixmap.create_from_xpm_d
      (window, _dove_gdk_mask, (GdkColor *)NULL, dove_xpm);

    _dove_pixmap.set(_dove_gdk_pixmap, _dove_gdk_mask);
  }
}

void Paloma::
layout_window(const vector<string> &cds) {
  set_title(_title);
  add(_box1);
  _box1.show();

  _menubar = _menu.get_menubar_widget("");
  assert(_menubar.get_object() != NULL);

#ifdef HAVE_CDAUDIO
  // Make a menu item for each CD in the list.
  if (!cds.empty()) {
    if (cds.size() == 1) {
      _menu.create_item("/File/Scan CD", "", "",
			ItemFactoryConnector<Paloma, string>(this, &menu_scan_cd, cds[0]));
    } else {
      for (int i = 0; i < cds.size(); i++) {
	_menu.create_item("/File/Scan " + get_basename(cds[i]), "", "",
			  ItemFactoryConnector<Paloma, string>(this, &menu_scan_cd, cds[i]));
      }
    }
    _menu.create_item("/File/sep", "", "<Separator>", 0);
  }
#endif

  _menu.create_item("/File/Import", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_import));
  _menu.create_item("/File/Export CD's", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_export_cds));
  _menu.create_item("/File/Export songs", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_export_songs));
  _menu.create_item("/File/sep", "", "<Separator>", 0);
  _menu.create_item("/File/Configure", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_configure));

  _menu.create_item("/File/Quit", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_quit));

  _menu.create_item("/Lookup/Artists", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_lookup_artists));
  _menu.create_item("/Lookup/CD's", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_lookup_cds));
  _menu.create_item("/Lookup/Songs", "", "",
		    ItemFactoryConnector<Paloma, int>(this, &menu_lookup_songs));

  // Also add any menu query items specified by the user.
  QueryResult qr = music_db->sql_command
    ("SELECT form_id, menu_item, accelerator FROM form_queries "
     "ORDER BY sequence");
  int num_rows = qr.get_num_rows();
  for (int i = 0; i < num_rows; i++) {
    int form_id = qr.get_cell(i, 0).get_int();
    string menu_item = qr.get_cell(i, 1).get_string();
    string accelerator = qr.get_cell(i, 2).get_string();
    _menu.create_item(menu_item, accelerator, "",
		      ItemFactoryConnector<Paloma, int>(this, &menu_begin_query, form_id));
  }

  _box1.pack_start(*_menubar.get_object(), false, false);
  _menubar.get_object()->show();

  if (_show_dove) {
    _box1.pack_start(_dove_pixmap, false, false);
    _dove_pixmap.show();
  }
}
