/* circularBuffer.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef CIRCULARBUFFER_H
#define CIRCULARBUFFER_H

#include <iostream.h>

// This class implements a FIFO queue on streaming char* data.  If it
// is not empty, a certain number of contiguous bytes may be extracted
// from it; if it is not full, a certain number of contiguous bytes
// may be written to it.

class CircularBuffer {
public:
  CircularBuffer(int max_size);
  ~CircularBuffer();

  // Returns the maximum number of characters the buffer will hold.
  int get_max_size() const;

  // Returns the number of characters currently in the buffer.
  int get_current_size() const;

  // Convenience functions.
  bool is_empty() const;
  bool is_full() const;
  double get_percent_full() const;


  // Returns the maximum number of contiguous characters available for
  // reading.  This number will be <= get_current_size().  If it is
  // less, read() and extract_characters() must be called repeatedly
  // to extract the entire contents of the buffer.
  int get_read_size() const;

  // This points to a block of data that begins at least
  // get_read_size() bytes of contiguous data in the buffer.
  const char *read() const;

  // Once the data has been read via read(), call extract_characters()
  // to actually remove the data from the buffer.  num_characters must
  // be <= get_read_size().
  void extract_characters(int num_characters);


  // The above three functions can be used to extract all the data
  // from the buffer in a loop something like this:
  //
  //  while (cb.get_current_size() > 0) {
  //    int chars_extracted = write(fd, cb.read(), cb.get_read_size());
  //    cb.extract_characters(chars_extracted);
  //  }


  // Returns the maximum number of contiguous characters that may be
  // written into the buffer.  This number will be <= (get_max_size()
  // - get_current_size()).  If it is less, write() and
  // insert_characters() must be called repeatedly to fill up the
  // buffer.
  int get_write_size() const;

  // This points to a block of data that begins at most
  // get_write_size() bytes of contiguous data which may be directly
  // written to.
  char *write();

  // Once the data has been filled in the block pointed to by
  // write(), call insert_characters() to actually add the data
  // to the buffer.  num_characters must be <=
  // get_write_size().
  void insert_characters(int num_characters);


  // The above three functions can be used to fill up the buffer in a
  // loop something like this:
  //
  //  while (cb.get_current_size() < cb.get_max_size()) {
  //    int chars_filled = read(fd, cb.write(), cb.get_write_size());
  //    cb.insert_characters(chars_filled);
  //  }


  // This function is just for handy debugging.
  void describe(ostream &out) const;

private:
  int _in;
  int _out;
  int _top;
  char *_buffer;
};

#endif

